// TRUE 3D image of my BFV12/97 Bacardi Ferrari V12 (c)1998 James Shaughnessy
//
// Unlike the smaller GIF on my page (where you look 'through' the screen)
// on this demo you must look in front of the screen by crossing your
// eyes.  So the left car is viewed by the right eye and vice versa.
// This is similar to Peter Passmore's cross-eyed fusion game demo which
// is worth checking out if you can actually 'do' this.
// It does hurt the old mince pies after a while so it's probably better
// you don't look at it too long.  It does take a couple of minutes for
// your brain to allow your eyes to focus past the virtual image, but it
// does all look nice and TRUE 3D when it locks on target.
//
// If you have no idea how these two cars manage to be true 3D and don't
// think you will ever get it focused while cross-eyed then try this:
// Sit about 2 feet away (more if bigger than a 14" screen) and then put
// your index finger about six inches from your nose, and look at that.
// You shoud see THREE blurry cars in the background.  Concentrate on the
// 'middle' one, and try and focus on it but leaving your eyes crossed
// (looking at your fingertip).  It takes practice to be able to see it
// right away, but it's worth persevering if you have trouble.
//
// Incidentally if you see my team page you'll notice that the tobacco
// sponsorship had to be 'filtered' out (ugh) and the 1998 car has livery
// of the sacred PSX logo.  Money ran out after the first race though.
// http://www.manc.u-net.com/gp2.htm
//
// Runs in 640x480 interlaced mode, but the gfx are aliased to minimise
// flicker (mucho like in the brilliant Timeshock! pinball game)
//
// Cheers, Jim                27th July 1998
// -----------------------------------------
// James Shaughnessy    james@manc.u-net.com
// http://www.netyaroze-europe.com/~shaughnj
// -----------------------------------------


#include <libps.h>
#include "pad.h"


#define TIM_ADDR (0x80090000)
#define OT_LENGTH (1)


// Global Variables

// Two Ordering Tables, one for each buffer
GsOT WorldOT[2];
// Two Ordering Table Tags, one for each buffer
GsOT_TAG OTTag[2][1<<OT_LENGTH];
//Also allocate memory used for depth sorting, a block for each buffer
PACKET GpuPacketArea[2][204800];

int activeBuff;
int hsyncs;
u_long padStatus = 0;
extern DISPENV GsDISPENV;

GsSPRITE Bacard3D[3];
RECT screen = {0, 0, 640, 480};


// Function Prototypes
void InitGraphics();
void InitSprites();


void main()
{

  PadInit();
  InitGraphics();

  InitSprites();

  while(1)
  {
    padStatus = PadRead();

    // Quit if both Select and Start pressed
    if (padStatus & PADselect && padStatus & PADstart) break;

    activeBuff = GsGetActiveBuff();

    // Set packet work base
    GsSetWorkBase((PACKET *)GpuPacketArea[activeBuff]);

    // Clear the Ordering Table
    GsClearOt(0, 0, &WorldOT[activeBuff]);


    // Sort sprites in OT
    GsSortFastSprite(&Bacard3D[0], &WorldOT[activeBuff], 0);
    GsSortFastSprite(&Bacard3D[1], &WorldOT[activeBuff], 0);
    GsSortFastSprite(&Bacard3D[2], &WorldOT[activeBuff], 0);

#ifdef DEBUG
    // Display H-Sync counter
    FntPrint("HSyncs: %d\n", hsyncs);
#endif
    FntFlush(-1);

    // Wait for end of drawing, a vertical blank then swap buffers
    DrawSync(0);
    hsyncs = VSync(0);

    // Clear screen to grey
    ClearImage(&screen, 140, 132, 140);

    GsSwapDispBuff();

    // Draw the Ordering Table
    GsDrawOt(&WorldOT[activeBuff]);
  }

  ResetGraph(0);
}


// Initialises Video and the Frame Buffer
void InitGraphics()
{
  int i;

  // Setup graphics PAL
  SetVideoMode(MODE_PAL);
  GsInitGraph(640, 480, 1, 0, 0);
  //GsDISPENV.screen.y = 20;
  //GsDISPENV.screen.h = 480;
  GsDefDispBuff(0, 0, 0, 0);

  // Init Ordering Tables
  for (i = 0; i < 2; i++) {
    WorldOT[i].length = OT_LENGTH;
    WorldOT[i].org = OTTag[i];
    GsClearOt(0, 0, &WorldOT[i]);
  }

  // Loads the font into Frame Buffer
  FntLoad(960, 256);
  // Font printing location
  FntOpen(40, 30, 192, 192, 0, 512);
}


void InitSprites()
{
  RECT r;
  GsIMAGE im;

  // Put TIM info into im
  GsGetTimInfo(((u_long *)TIM_ADDR)+1, &im);
  // Load image and clut into frame buffer
  r.x = im.px;
  r.y = im.py;
  r.w = im.pw;
  r.h = im.ph;
  LoadImage(&r, im.pixel);
  r.x = im.cx;
  r.y = im.cy;
  r.w = im.cw;
  r.h = im.ch;
  LoadImage(&r, im.clut);

  DrawSync(0);

  Bacard3D[0].attribute = (im.pmode<<24);
  Bacard3D[0].x = 0;
  Bacard3D[0].y = 144;
  Bacard3D[0].w = 256;
  Bacard3D[0].h = 192;
  Bacard3D[0].tpage = GetTPage(im.pmode, 0, 640, 0);
  Bacard3D[0].cx = im.cx;
  Bacard3D[0].cy = im.cy;
  Bacard3D[0].u = 0;
  Bacard3D[0].v = 0;
  Bacard3D[0].r = Bacard3D[0].g = Bacard3D[0].b = 128;
  Bacard3D[0].scalex = ONE;
  Bacard3D[0].scaley = ONE;
  Bacard3D[0].rotate = 0;
  Bacard3D[1].attribute = (im.pmode<<24);
  Bacard3D[1].x = 256;
  Bacard3D[1].y = 144;
  Bacard3D[1].w = 256;
  Bacard3D[1].h = 192;
  Bacard3D[1].tpage = GetTPage(im.pmode, 0, 768, 0);
  Bacard3D[1].cx = im.cx;
  Bacard3D[1].cy = im.cy;
  Bacard3D[1].u = 0;
  Bacard3D[1].v = 0;
  Bacard3D[1].r = Bacard3D[1].g = Bacard3D[1].b = 128;
  Bacard3D[1].scalex = ONE;
  Bacard3D[1].scaley = ONE;
  Bacard3D[1].rotate = 0;
  Bacard3D[2].attribute = (im.pmode<<24);
  Bacard3D[2].x = 512;
  Bacard3D[2].y = 144;
  Bacard3D[2].w = 128;
  Bacard3D[2].h = 192;
  Bacard3D[2].tpage = GetTPage(im.pmode, 0, 896, 0);
  Bacard3D[2].cx = im.cx;
  Bacard3D[2].cy = im.cy;
  Bacard3D[2].u = 0;
  Bacard3D[2].v = 0;
  Bacard3D[2].r = Bacard3D[2].g = Bacard3D[2].b = 128;
  Bacard3D[2].scalex = ONE;
  Bacard3D[2].scaley = ONE;
  Bacard3D[2].rotate = 0;
}




// ****** PAD routines ******

// call once only in program initialisation
void PadInit (void)
{
  GetPadBuf(&bb0, &bb1);
}

// call once per VSync(0)
// puts controller pad status into unsigned long integer
// please refer to the manuals if you want explanation 
// of the internals of this function
u_long PadRead(void)
{
  return(~(*(bb0+3) | *(bb0+2) << 8 | *(bb1+3) << 16 | *(bb1+2) << 24));
}

